
/*

    File: chdir.c

    Copyright (C) 1998,1999  Wolfgang Zekoll  <wzk@quietsche-entchen.de>
    Copyright (c) 2000,2002,2003  Andreas Schoenberg  <asg@pce.de>
  
    This software is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
  
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
  
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

 */


#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <stdarg.h>
#include <errno.h>

#include <time.h>
#include <signal.h>
#include <wait.h>
#include <ctype.h>
#include <pwd.h>

#include <sys/types.h>
#include <sys/stat.h>
#include <sys/fcntl.h>
#include <sys/socket.h>
#include <netdb.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <syslog.h>
#include <sys/time.h>

#include "ftp.h"
#include "ip-lib.h"
#include "lib.h"

#define	DIR_MAXDEPTH		15


char *get_ftppath(ftp_t *x, char *path, int size)
{
	int	len;

	len = strlen(x->rootdir);
	if (strncmp(path, x->rootdir, len) != 0  ||  (path[len] != '/'  &&  path[len] != 0)) {
		syslog(LOG_NOTICE, "-ERR: path out of virtual root: %s, virtual root= %s", path, x->rootdir);
		exit (1);
		}
	else {
		copy_string(path, &path[len], size);
		if (*path == 0)
			strcpy(path, "/");
		}

	return (path);
}

char *get_syspath(ftp_t *x, char *path, int size)
{
	char	dir[200];

	if (*path != '/')
		*path = 0;
	else {
		snprintf (dir, sizeof(dir) - 2, "%s%s", x->rootdir, path);
		copy_string(path, dir, size);
		}

	return (path);
}

int normalize_path(ftp_t *x, char *path, int size)
{
	int	i, k, n, m;
	char	cwp[200], ftpdir[200], *part[20], *dir[20];

	/*
	 * Zuerst wird das aktuelle Verzeichnis (der ftppath) in seine
	 * Einzelteile zerlegt ...
	 */

	if (*path == '/') {

		/*
		 * ... Ausnahme: die path-Angabe ist absolut ...
		 */

		dir[0] = "";
		n = 1;
		}
	else {
		getcwd(cwp, sizeof(cwp));
		copy_string(ftpdir, &cwp[strlen(x->rootdir)], sizeof(ftpdir));
		if (*ftpdir != 0  &&  strcmp(ftpdir, "/") != 0)
			n = split(ftpdir, part, '/', DIR_MAXDEPTH);
		else {
			dir[0] = "";
			n = 1;
			}
		}

	/*
	 * ... danach der path.  Die path Teile werden unter Beachtung
	 * der ueblichen Regeln an die Teile des aktuellen Verzeichnisses
	 * angehangen ...
	 */

	m = split(path, dir, '/', 15);
	for (i=0; i<m; i++) {
		if (*dir[i] == 0)
			/* nichts */ ;
		else if (strcmp(dir[i], ".") == 0)
			/* nichts */ ;
		else if (strcmp(dir[i], "..") == 0) {
			if (n > 1)
				n = n - 1;
			}
		else if (0  &&  dir[i][0] == '.')
			return (1);		/* kein Zugriff */
		else
			part[n++] = dir[i];

		if (n < 1  ||  n >= DIR_MAXDEPTH)
			return (1);		/* ungueltiges Verzeichnis */
		}

	/*
	 * ... und das Ergebnis wieder zusammengesetzt.
	 */

	if (n <= 1) {
		strcpy(cwp, "/");
		}
	else {
		k = 0;
		for (i=1; i<n; i++) {
			if (*part[i] == 0)
				continue;

			if ((k + strlen(part[i]) + 1 + 2) >= sizeof(cwp))
				return (1);		/* Name zu lang */
				
			cwp[k++] = '/';
			strcpy(&cwp[k], part[i]);
			k += strlen(&cwp[k]);
			}

		cwp[k] = 0;
		}

	/*
	 * Der normalisierte path auf das Objekt (Datei oder Verzeichnis,
	 * ist hier egal) ist fertig.
	 */

	copy_string(path, cwp, size);
	return (0);
}


char *getdir(char *path, char *dir, int size)
{
	char	*p;

	copy_string(dir, path, size);
	if ((p = strrchr(dir, '/')) != NULL  &&  p != dir)
		*p = 0;

	return (dir);
}

char *getfile(char *path, char *file, int size)
{
	char	*p;

	*file = 0;
	if ((p = strrchr(path, '/')) != NULL  &&  p[1] != 0)
		copy_string(file, &p[1], size);

	return (file);
}


