
/*

    File: ftp.h

    Copyright (C) 1998,1999  Wolfgang Zekoll  <wzk@quietsche-entchen.de>
    Copyright (c) 2000,2002,2003  Andreas Schoenberg  <asg@pce.de>
  
    This software is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
  
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
  
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

 */


#ifndef	_FTP_INCLUDED
#define	_FTP_INCLUDED


extern char *program;
extern char progname[80];

extern int debug;


typedef struct _config {
    int		timeout;

    char	acp[200];
    char	ccp[200];
    char	handler[200];

    unsigned long umask;
    char	vroot[200];
    char	homedir[200];
    struct {
	unsigned int uid;
	unsigned int gid;
	} user;

    int		acp_authenticates;
    int		allow_blanks;
    int		allow_quotes;
    int		allow_anyremote;
    int		ignore_anyremote;
    int		sgphack;

    char	denylist[200];

    int		allow_anon;
    char	anon_user[80];

    char	varname[80];
    char	sitehandler[200];
    char	unamehandler[200];

/*    char	tmpfile[200]; */
    } config_t;


#define	HANDLER_OK		0
#define	HANDLER_ERROR		-1


#define	FTPBUFLEN		2048

#define	TYPE_BIN		0
#define	TYPE_ASC		1

#define PORT_UNCONNECTED	0
#define	PORT_LISTEN		1
#define	PORT_CONNECTED		2
#define	PORT_CLOSED		3

#define	MODE_PORT		1
#define	MODE_PASSIVE		2	/* PASV wird nicht unterstuetzt */

#define	OP_GET			1
#define	OP_PUT			2


typedef struct _port {
    char	ipnum[80];
    unsigned int port;
    } port_t;

typedef struct _dtc {
    int		state;		/* PORT_UNCONNECTED, _LISTEN, _CONNECTED, _CLOSED */
    
    int		local;
    int		data;
    int		localpipe;
    
    int		operation;	/* GET oder PUT */
    int		active;		/* fd auf sender */
    int		other;		/* fd auf receiver */

    int		type;		/* ASC oder BIN */
    int		mode;		/* PORT oder PASV */

    port_t	pasv;		/* unser Datenport im Passive-Mode */
    port_t	client;


    char	command[20];	/* Fuer syslog Meldungen */
    char	filename[200];

    unsigned long bytes;
    } dtc_t;


#define	CCP_OK			0
#define	CCP_ERROR		1


typedef struct _ftp {
    config_t	*config;

    char	interface[80];
    unsigned int port;

    char	client[200];
    char	client_ip[80];

    char	username[200];
    char	password[200];

    char	gecos[100];
    char	email[80];

    char	rootdir[200];
    char	home[200];
    char	dir[200];

    int		type;		/* ASC oder BIN */
    unsigned long restart;

    dtc_t	ch;
    char	renamefile[200];
    char	sitepar[300];
    char	origpar[300];


    struct {
	int		lastfd;
	int		here, len;
	char		buffer[4096];
	} bio;

    int		commands;
    unsigned long datain, dataout;
    } ftp_t;



extern char *get_ftppath(ftp_t *x, char *path, int size);
extern char *get_syspath(ftp_t *x, char *path, int size);
extern int normalize_path(ftp_t *x, char *path, int size);
extern char *getdir(char *path, char *dir, int size);
extern char *getfile(char *path, char *file, int size);


extern int listdir(ftp_t *x, char *path, int mode);
extern int getuniquename(ftp_t *x, char *filename, int size);
extern int dosite(ftp_t *x, char *par);


extern int cfputs(ftp_t *x, char *line);
extern int setvar(ftp_t *x, char *var, char *value);
extern int set_variables(ftp_t *x, char *cmd, char *par, int ispath);

extern int call_handler(ftp_t *x, char *cmd, char *par, int ispath);
extern int ftp_session(config_t *config);

#endif

