/*

    File: lib.c
    
    Copyright (C) 1998,1999  Wolfgang Zekoll  <wzk@quietsche-entchen.de>
    Copyright (c) 2000,2002,2003  Andreas Schoenberg  <asg@pce.de>

    This source is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 1, or (at your option)
    any later version.

    This source is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <unistd.h>
#include <time.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/stat.h>

#include "lib.h"

#define	DEBUG(x)		


void *allocate(size_t size)
{
	void	*p;

	if ((p = malloc(size)) == NULL) {
		fprintf (stderr, "%s: memory allocation error\n", program);
		exit (-1);
		}

	memset(p, 0, size);
	return (p);
}

void *reallocate(void *p, size_t size)
{
	if ((p = realloc(p, size)) == NULL) {
		fprintf (stderr, "%s: memory allocation error\n", program);
		exit (-1);
		}

	return (p);
}



static unsigned int lower[256], upper[256];

int _init_upper()
{
	unsigned int c;
	
	DEBUG( fprintf (stderr, "init upper[]\n"); )
	for (c = 0; c < 256; c++)
		upper[c] = c;

	DEBUG( fprintf (stderr, "init uppercase letters\n"); )
	for (c = 'a'; c < 'z'+1; c++)
		upper[c] = 'A' + (c - 'a');

	DEBUG( fprintf (stderr, "init umlaute\n"); )
	upper[c = (unsigned char) ''] = '';
	upper[c = (unsigned char) ''] = '';
	upper[c = (unsigned char) ''] = '';

	DEBUG( fprintf (stderr, "init upper[] complete\n"); )
	return (0);
}

int _init_lower()
{
	unsigned int c;
	
	DEBUG( fprintf (stderr, "init lower[]\n"); )
	for (c = 0; c < 256; c++)
		lower[c] = c;

	DEBUG( fprintf (stderr, "init uppercase letters\n"); )
	for (c = 'A'; c < 'Z'+1; c++)
		lower[c] = 'a' + (c - 'A');

	DEBUG( fprintf (stderr, "init umlaute\n"); )
	lower[c = (unsigned char) ''] = '';
	lower[c = (unsigned char) ''] = '';
	lower[c = (unsigned char) ''] = '';

	DEBUG( fprintf (stderr, "init upper[] complete\n"); )
	return (0);
}


unsigned int uppercase(unsigned int c)
{
	if (upper['0'] == 0)
		_init_upper();

	return (upper[c]);
}

int isuppercase(unsigned int c)
{
	if (upper['0'] == 0)
		_init_upper();

	return (upper[c] == c);
}

unsigned int lowercase(unsigned int c)
{
	if (lower['0'] == 0)
		_init_lower();

	return (lower[c]);
}

int islowercase(unsigned int c)
{
	if (lower['0'] == 0)
		_init_lower();

	return (lower[c] == c);
}

char *strlwr(char *string)
{
	unsigned int c;
	unsigned char *p;

	if (lower['0'] == 0)
		_init_lower();
		
	p = string;
	while ((c = *p) != 0) {
		*p++ = lower[c];
		}

	return (string);
}	

char *strupr(char *string)
{
	unsigned int c;
	unsigned char *p;

	if (upper['0'] == 0)
		_init_upper();
		
	p = string;
	while ((c = *p) != 0) {
		*p++ = upper[c];
		}

	return (string);
}	

unsigned int get_stringcode(char *string)
{
	int	i, len, code;

	code = 0;
	len  = strlen(string);
	for (i=0; i<len; i++) {
		code = code + string[i];
		}

	code = (code & 0XFF) | (len << 8);
	return (code);
}


     /* -------------------------------------------------------------
      */

char *skip_ws(char *string)
{
	unsigned int c;

	while ((c = *string) == ' '  ||  c == '\t')
		string++;

	return (string);
}

char *noctrl(char *buffer)
{
	int	len, i;
	unsigned char *p;

	if ((p = buffer) == NULL)
		return (NULL);

	len = strlen(p);
        for (i=len-1; i>=0; i--) {
		if (p[i] <= 32)
			p[i] = '\0';
		else
			break;
		}

	return (p);
}

char *get_word(char **from, char *to, int maxlen)
{
	unsigned int c;
	unsigned char *p;
	int	k;

	maxlen -= 2;
	while ((c = **from) != 0  &&  c <= 32)
		*from += 1;

	*(p = to) = k = 0;
	while ((c = **from) != 0) {
		if (c == ' '  ||  c == '\t'  ||  c < 32)
			break;

		*from += 1;
		if (k < maxlen)
			p[k++] = c;
		}

	p[k] = 0;
	return (to);
}

char *get_quoted(char **from, int delim, char *to, int max)
{
	unsigned int c;
	int	k;

	to[0] = k = 0;
	max -= 2;
	
	while ((c = **from) != 0) {
		*from += 1;
		if (c == delim)
			break;

		if (k < max)
			to[k++] = c;
		}

	to[k] = 0;
	return (to);
}

char *get_escquoted(char **from, int esc, int delim, int unesc, char *to, int max)
{
	unsigned int c;
	int	k;

	to[0] = k = 0;
	max -= 2;
	
	while ((c = **from) != 0) {
		*from += 1;
		if (c == esc  &&  **from == delim) {
			if (unesc == 0)
				to[k++] = c;

			to[k++] = delim;
			*from += 1;
			}
		else if (c == delim)
			break;
		else if (k < max)
			to[k++] = c;
		}

	to[k] = 0;
	return (to);
}

char *get_string(char **from, char *to, int max)
{
	unsigned int c, d;
	int	k;

	to[0] = k = 0;
	while ((c = **from) != 0  &&  c <= ' ')
		*from += 1;

	if (c == 0)
		return (to);
	else if (c != '"')
		return (get_word(from, to, max));

	*from += 1;
	max -= 2;
	d = c;
	while ((c = **from) != 0) {
		*from += 1;
		if (c == d)
			break;

		if (k < max)
			to[k++] = c;
		}

	to[k] = 0;
	return (to);
}

char *get_parenthesis(char **from, int open, int close, char *string, int max)
{
	int	c, d, k, level;

	string[0] = k = 0;
	max -= 2;
	
	level = 0;
	while ((c = **from) != 0  &&  c != '\n') {
		*from += 1;
		
		if (c == open)
			level += 1;
		else if (c == close) {
			level -= 1;
			if (level < 0)
				break;
			}

		if (c == '\\'  &&  ((d = **from) == open   ||  d == close)) {
			*from += 1;
			if (k < max)
				string[k] = d;
			}
		else if (k < max)
			string[k++] = c;
		}

	string[k] = 0;
	return (string);
}

int split(char *line, char *ptr[], int sep, int max)
{
	char	*p;
	int	k, c;

	if (line[0] == 0) {
		ptr[0] = line;
		return (0);
		}

	p = line;
	k = 0;
	while (*p != 0	&&  k < max) {
		if (sep == ' ') {
			while ((c = *p) == ' '	||  c == '\t')
				p++;
			}

		ptr[k++] = p;
		if (sep == ' ') {
			while ((c = *p) != 0  &&  c != '\t'  &&  c != ' ')
				p++;
			}
		else {
			while ((c = *p) != 0  &&  c != sep)
				p++;
			}

		if (*p != 0)
			*p++ = 0;
		}

	return (k);
}



     /* -------------------------------------------------------------
      */

int is_file(char *filename)
{
	struct stat sbuf;

	if (stat(filename, &sbuf) == -1)
		return (-1);

	if ((sbuf.st_mode & S_IFMT) == S_IFREG)
		return (0);

	return (-1);
}

unsigned long get_mtime(char *filename)
{
	struct stat sbuf;

	if (stat(filename, &sbuf) == -1)
		return (0);

	return (sbuf.st_mtime);
}

int is_dir(char *filename)
{
	struct stat sbuf;

	if (stat(filename, &sbuf) == -1)
		return (-1);

	if ((sbuf.st_mode & S_IFMT) == S_IFDIR)
		return (0);

	return (-1);
}

int change_dir(char *old, char *new)
{
	int	rc;

	rc = 0;
	if (old != NULL) {
		getcwd(old, 200);
		strlwr(old);
		}

	if (new != NULL  &&  new[0] != 0) {
		rc = chdir(new);
		}

	return (rc);
}


char *extract_name(char *name, char *filename)
{
	int	i, k, len;

	*name = 0;
	len = strlen(filename);
	for (k = len-1; k >= 0; k--) {
		if (filename[k] == '/')
			break;
		}

	if (k < 0) {
		strcpy(name, filename);
		return (name);
		}

	while (filename[k] == '/')
		k++;

	for (i=0; filename[k] != 0; i++, k++)
		name[i] = filename[k];

	name[i] = 0;
	return (name);
}

char *extract_dir(char *dir, char *filename)
{
	int	k;

	strcpy(dir, filename);
	for (k = strlen(dir)-1; k >= 0; k--) {
		if (dir[k] == '/') {
			dir[k] = 0;
			return (dir);
			}
		}

	dir[0] = 0;
	return (dir);
}

char *trunc_extension(char *filename)
{
	int	c, k, len;

	len = strlen(filename);
	for (k = len-1; k >= 0; k--) {
		if ((c = filename[k]) == '.') {
			filename[k] = 0;
			break;
			}
		else if (c == '/')
			break;
		}

	return (filename);
}


     /* -------------------------------------------------------------
      * string Funktionen
      */

	/*
	 * Taken from the wn sources (see wild_match()) - strpcmp()
	 * make case-insensitive comparisons.
	 */
	 
int strpcmp(char *string, char *pattern)
{
	if (*string == 0) {
		if (*pattern == 0)
			return (0);
		else if (*pattern == '*')
			return (strpcmp(string, pattern + 1));
		else
			return (1);
		}

	if (*pattern == '*') {
		while (strpcmp(string, pattern + 1) != 0) {
			string++;
			if (*string == 0) {
				if (*(pattern + 1) != 0)
					return (1);
				else
					return (0);
				}
			}

		return (0);
		}
	else if (*pattern == '?')
		return (strpcmp(string + 1, pattern + 1));
	else if (tolower(*string) == tolower(*pattern))
		return (strpcmp(string + 1, pattern + 1));

	return (1);
}

char *copy_string(char *y, char *x, int len)
{
	x = skip_ws(x);
	noctrl(x);

	len -= 2;
	if (strlen(x) >= len)
		x[len] = 0;

	if (y != x)
		strcpy(y, x);
		
	return (y);
}

