
/*

    File: listdir.c

    Copyright (C) 1998,1999  Wolfgang Zekoll  <wzk@quietsche-entchen.de>
    Copyright (c) 2000,2002,2003  Andreas Schoenberg  <asg@pce.de>
  
    This software is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
  
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
  
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

 */


#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <stdarg.h>
#include <errno.h>

#include <time.h>
#include <signal.h>
#include <wait.h>
#include <ctype.h>

#include <pwd.h>
#include <grp.h>

#include <sys/types.h>
#include <sys/stat.h>
#include <sys/fcntl.h>
#include <sys/socket.h>
#include <netdb.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <syslog.h>
#include <sys/time.h>
#include <dirent.h>

#include "ftp.h"
#include "ip-lib.h"
#include "lib.h"


#define	ONE_MONTH		(60 * 60 * 24 * 30L)

static char filepattern[80]	= "*";


typedef struct {
    unsigned int id;
    char	username[12];
    char	groupname[12];
    } idmap_t;

static idmap_t *idmap =		NULL;
static int idn =		0;
static int idmax =		0;



static char *getdirname(char *path, char *dir, int size)
{
	char	*p;

	copy_string(dir, path, size);
	if ((p = strrchr(dir, '/')) != NULL)
		*p = 0;

	if (*dir == 0)
		copy_string(dir, "/", size);

	return (dir);
}


static char *getname(unsigned int id, int type)
{
	int	i;
	char	*name;
	idmap_t	*ide;

	for (i=0; i<idn; i++) {
		if (idmap[i].id == id)
			break;
		}

	if (i < idn)
		ide = &idmap[i];
	else {
		if (idn >= idmax) {
			idmax += 10;
			idmap = reallocate(idmap, sizeof(idmap_t) * idmax);
			}

		ide = &idmap[idn++];

		ide->id = id;
		*ide->username = 0;
		*ide->groupname = 0;
		}


	if (type == 0)
		name = ide->username;
	else
		name = ide->groupname;

	if (*name == 0) {
		if (type == 0) {
			struct passwd *pw;

			if ((pw = getpwuid(id)) == NULL)
				snprintf (ide->username, sizeof(ide->username) - 2, "%d", id);
			else
				copy_string(ide->username, pw->pw_name, sizeof(ide->username));

			ide->username[8] = 0;
			}
		else {
			struct group *gp;

			if ((gp = getgrgid(id)) == NULL)
				snprintf (ide->groupname, sizeof(ide->groupname) - 2, "%d", id);
			else
				copy_string(ide->groupname, gp->gr_name, sizeof(ide->groupname));

			ide->groupname[8] = 0;
			}

		}

	return (name);
}

static int listfile(char *path, char *name)
{
	char	perm[20], *user, *group, date[40];
	struct stat sbuf;
	struct tm lt;
	static time_t now = 0;

	if (now == 0)
		now = time(NULL);

	if (stat(path, &sbuf) != 0)
		return (1);


	perm[0] = (S_ISDIR(sbuf.st_mode))? 'd': '-';
	
	perm[1] = (sbuf.st_mode & S_IRUSR)? 'r': '-';
	perm[2] = (sbuf.st_mode & S_IWUSR)? 'w': '-';
	perm[3] = (sbuf.st_mode & S_IXUSR)? 'x': '-';
	
	perm[4] = (sbuf.st_mode & S_IRGRP)? 'r': '-';
	perm[5] = (sbuf.st_mode & S_IWGRP)? 'w': '-';
	perm[6] = (sbuf.st_mode & S_IXGRP)? 'x': '-';
	
	perm[7] = (sbuf.st_mode & S_IROTH)? 'r': '-';
	perm[8] = (sbuf.st_mode & S_IWOTH)? 'w': '-';
	perm[9] = (sbuf.st_mode & S_IXOTH)? 'x': '-';
	
	perm[10] = 0;


	user = getname(sbuf.st_uid, 0);
	group = getname(sbuf.st_gid, 1);

	lt = *localtime(&sbuf.st_mtime);
	if ((now - sbuf.st_mtime) > ONE_MONTH)
		strftime(date, sizeof(date) - 2, "%b %d  %Y", &lt);
	else
		strftime(date, sizeof(date) - 2, "%b %d %H:%M", &lt);

	printf ("%s %3d %-8s %-8s %10ld %s %s\n",
		perm, sbuf.st_nlink, user, group, sbuf.st_size, date, name);
	return (0);
}

int simplematch(const char *filename, char *pattern)
{
	if (*filename == 0) {
		if (*pattern == 0)
			return (1);
		else if (*pattern == '*')
			return (simplematch(filename, pattern + 1));
		else
			return (0);
		}

	if (*pattern == '*') {
		while (simplematch(filename, pattern + 1) == 0) {
			filename++;
			if (*filename == 0) {
				if (*(pattern + 1) != 0)
					return (0);
				else
					return (1);
				}
			}

		return (1);
		}
	else if (*pattern == '?')
		return (simplematch(filename + 1, pattern + 1));
	else if (*filename == *pattern)
		return (simplematch(filename + 1, pattern + 1));

	return (0);
}

int selectfile(const struct dirent *file)
{
	if (*filepattern == 0)
		return (1);

	return (simplematch(file->d_name, filepattern));
}

static int list(ftp_t *x, char *path, int mode)
{
	int	i, n;
	char	*name, dir[200], pattern[80], filepath[200], nlistdir[200];
	struct dirent **namelist;
	struct stat sbuf;

	if (mode == 0)
		printf ("total 7\r\n");

	/*
	 * Wir zerlegen den LIST/NLST Parameter in den Verzeichnisanteil
	 * und ein ggf. vorhandenes Filepattern.  Parallel dazu berechnen
	 * wir das Verzeichnis, so dass es zum Original Befehlsparameter
	 * passt.  Bei NLST muessen wir naemlich ggf. den Verzeichnisnamen
	 * so ausgeben das der Client ihn versteht.
	 */

	*nlistdir = 0;
	if (stat(path, &sbuf) == 0  &&  S_ISDIR(sbuf.st_mode)) {
		copy_string(dir, path, sizeof(dir));
		strcpy(pattern, "*");
		copy_string(nlistdir, x->origpar, sizeof(nlistdir));
		}
	else if (strrchr(path, '/') == path) {
		strcpy(dir, "/");
		getfile(path, pattern, sizeof(pattern));
		}
	else {
/*		getdir(path, dir, sizeof(dir)); */
		getdirname(path, dir, sizeof(dir));
		getfile(path, pattern, sizeof(pattern));
/*		getdir(x->origpar, nlistdir, sizeof(nlistdir)); */
		getdirname(x->origpar, nlistdir, sizeof(nlistdir));
		}

	copy_string(filepattern, pattern, sizeof(filepattern));
	n = scandir(dir, &namelist, selectfile, alphasort);
	if (n < 0) {
		syslog(LOG_NOTICE, "scandir error: %s, error= %s", path, sys_errlist[errno]);
		return (1);
		}

	if (strcmp(dir, "/") == 0)
		*dir = 0;

	for (i=0; i<n; i++) {
		name = (*namelist[i]).d_name;
		if (strcmp(name, ".") == 0  ||  strcmp(name, "..") == 0)
			continue;

		if (mode == 1) {
			if (*nlistdir == 0)
				printf ("%s\n", name);
			else if (nlistdir[0] == '/'  &&  nlistdir[1] == 0)
				printf ("/%s\n", name);
			else
				printf ("%s/%s\n", nlistdir, name);
			}
		else {
			snprintf (filepath, sizeof(filepath) - 2, "%s/%s", dir, (*namelist[i]).d_name);
			listfile(filepath, (*namelist[i]).d_name);
			}
		}

	return (0);
}


int listdir(ftp_t *x, char *path, int mode)
{
	int	rc, pid, pfd[2];

	rc = 0;
	if (pipe(pfd) != 0) {
		syslog(LOG_NOTICE, "-ERR: can't pipe: %m");
		exit (1);
		}
	else if ((pid = fork()) < 0) {
		syslog(LOG_NOTICE, "-ERR: can't fork listdir: %m");
		exit (1);
		}
	else if (pid == 0) {
		dup2(pfd[1], 1);
		close(pfd[0]);

		list(x, path, mode);
		exit (0);
		}

	close(pfd[1]);
	return (pfd[0]);
}

