
/*

    File: main.c

    Copyright (C) 1998,1999  Wolfgang Zekoll  <wzk@quietsche-entchen.de>
    Copyright (c) 2000,2002,2003  Andreas Schoenberg  <asg@pce.de>

    This software is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
  
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
  
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

 */


#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <stdarg.h>
#include <ctype.h>

#include <signal.h>
#include <wait.h>
#include <pwd.h>

#include <sys/types.h>
#include <sys/stat.h>
#include <sys/fcntl.h>
#include <sys/socket.h>
#include <netdb.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <syslog.h>
#include <sys/time.h>

#include "ftp.h"
#include "ip-lib.h"
#include "lib.h"


char	*program =		"";
char	progname[80] =		"";

int	debug =			0;

int	uid =			-1;
int	gid =			-2;

int	daemonmode =		0;
unsigned int bindport =		21;


static int get_yesno(char **from, char *par, char *filename, int lineno)
{
	char	word[80];

	if (**from == 0) {
		fprintf (stderr, "%s: missing parameter: %s, %s:%d\n", program, par, filename, lineno);
		exit (1);
		}

	get_word(from, word, sizeof(word));
	if (strcmp(word, "yes") == 0)
		return (1);
	else if (strcmp(word, "no") == 0)
		return (0);

	fprintf (stderr, "%s: bad parameter value: %s, parameter= %s, %s:%d\n", program, word, par, filename, lineno);
	exit (1);
	
	return (0);
}

static char *get_parameter(char **from, char *par, char *value, int size,
		char *filename, int lineno)
{
	if (**from == 0) {
		fprintf (stderr, "%s: missing parameter: %s, %s:%d\n", program, par, filename, lineno);
		exit (1);
		}

	copy_string(value, *from, size);
	return (value);
}

int readconfig(config_t *config, char *filename)
{
	int	lineno;
	char	*p, word[80], line[300];
	FILE	*fp;

	if ((fp = fopen(filename, "r")) == NULL) {
		fprintf (stderr, "%s: can't open configuration file: %s\n", program, filename);
		exit (1);
		}

	lineno = 0;
	while (fgets(line, sizeof(line), fp) != NULL) {
		lineno++;
		if ((p = strchr(line, '#')) != NULL)
			*p = 0;

		p = skip_ws(noctrl(line));
		if (*p == 0)
			continue;

		get_word(&p, word, sizeof(word));
		strlwr(word);
		p = skip_ws(p);

		if (strcmp(word, "debug") == 0)
			debug = get_yesno(&p, word, filename, lineno);
		
		else if (strcmp(word, "acp") == 0)
			get_parameter(&p, word, config->acp, sizeof(config->acp), filename, lineno);
		else if (strcmp(word, "acp-auth") == 0)
			config->acp_authenticates = get_yesno(&p, word, filename, lineno);
		else if (strcmp(word, "ccp") == 0)
			get_parameter(&p, word, config->ccp, sizeof(config->ccp), filename, lineno);
		else if (strcmp(word, "handler") == 0)
			get_parameter(&p, word, config->handler, sizeof(config->handler), filename, lineno);

		else if (strcmp(word, "site") == 0  ||  strcmp(word, "site-handler") == 0)
			get_parameter(&p, word, config->sitehandler, sizeof(config->sitehandler), filename, lineno);
		else if (strcmp(word, "uname") == 0  ||  strcmp(word, "uname-handler") == 0)
			get_parameter(&p, word, config->unamehandler, sizeof(config->unamehandler), filename, lineno);

		else if (strcmp(word, "rootdir") == 0)
			get_parameter(&p, word, config->vroot, sizeof(config->vroot), filename, lineno);
		else if (strcmp(word, "homedir") == 0)
			get_parameter(&p, word, config->homedir, sizeof(config->homedir), filename, lineno);

		else if (strcmp(word, "allow-blanks") == 0)
			config->allow_blanks = get_yesno(&p, word, filename, lineno);
		else if (strcmp(word, "allow-anon") == 0  ||  strcmp(word, "allow-anonymous") == 0)
			config->allow_anon = get_yesno(&p, word, filename, lineno);
		else if (strcmp(word, "anon-user") == 0)
			get_parameter(&p, word, config->anon_user, sizeof(config->anon_user), filename, lineno);

		else if (strcmp(word, "setenv") == 0  ||  strcmp(word, "varname") == 0)
			get_parameter(&p, word, config->varname, sizeof(config->varname), filename, lineno);
		else if (strcmp(word, "denylist") == 0)
			get_parameter(&p, word, config->denylist, sizeof(config->denylist), filename, lineno);
		else if (strcmp(word, "timeout") == 0) {
			char	val[20];

			get_parameter(&p, word, val, sizeof(val), filename, lineno);
			config->timeout = strtoul(val, NULL, 8);
			if (config->timeout < 60)
				config->timeout = 60;
			}
		else if (strcmp(word, "umask") == 0) {
			char	mask[20];

			get_parameter(&p, word, mask, sizeof(mask), filename, lineno);
			config->umask = strtoul(mask, NULL, 8);
			}

		else if (strcmp(word, "user") == 0) {
			if (isalpha(*p)) {
				struct passwd *pw;

				if ((pw = getpwnam(p)) == NULL) {
					fprintf (stderr, "%s: no such user: %s, %s:%d\n", program, p, filename, lineno);
					exit (1);
					}

				config->user.uid = pw->pw_uid;
				config->user.gid = pw->pw_gid;
				}
			else {
				char	*q;

				if (*p == '.') {
					config->user.uid = 65535U;
					q = p;
					}
				else
					config->user.uid = strtoul(p, &q, 10);

				if (*q == 0)
					config->user.gid = 65534U;
				else if (*q == '.') {
					q++;
					config->user.gid = strtoul(q, &q, 10);
					}
				else {
					fprintf (stderr, "%s: error in user id: %s, %s:%d\n", program, p, filename, lineno);
					exit (1);
					}
				}
			}
		else {
			fprintf (stderr, "%s: unknown parameter: %s, %s:%d\n", program, word, filename, lineno);
			exit (1);
			}
		}
		
	fclose (fp);
	return (0);
}


void missing_arg(int c, char *string)
{
	fprintf (stderr, "%s: missing arg: -%c, %s\n", program, c, string);
	exit (-1);
}

int main(int argc, char *argv[], char *envp[])
{
	int	c, i, k;
	char	*p, option[80];
	config_t *config;
	

	if ((p = strrchr(argv[0], '/')) == NULL)
		program = argv[0];
	else {
		copy_string(progname, &p[1], sizeof(progname));
		program = progname;
		}

	config = allocate(sizeof(config_t));
	config->timeout = 60 * 60;
	config->umask   = 0755;
	strcpy(config->varname, "FTP_");

	openlog(program, LOG_PID, LOG_MAIL);

	k = 1;
	while (k < argc  &&  argv[k][0] == '-'  &&  argv[k][1] != 0) {
		copy_string(option, argv[k++], sizeof(option));
		for (i=1; (c = option[i]) != 0; i++) {
			if (c == 'd')
				debug = 1;
			else if (c == 'a') {
				if (*config->acp != 0)
					config->acp_authenticates = 1;
				else {
					if (k >= argc)
						missing_arg(c, "access control program");

					copy_string(config->acp, argv[k++], sizeof(config->acp));
					}
				}
			else if (c == 'b') {
				if (config->allow_blanks != 0)
					config->allow_quotes = 1;
				else
					config->allow_blanks = 1;
				}
			else if (c == 'c') {
				if (k >= argc)
					missing_arg(c, "command control program");

				copy_string(config->ccp, argv[k++], sizeof(config->ccp));
				}
			else if (c == 'e') {
				if (config->allow_anyremote != 0)
					config->ignore_anyremote = 1;

				config->allow_anyremote = 1;
				}
			else if (c == 'f') {
				if (k >= argc)
					missing_arg(c, "configuration file");

				readconfig(config, argv[k++]);
				}
			else if (c == 'm') {
				if (k >= argc)
					missing_arg(c, "umask");

				config->umask = strtoul(argv[k++], &p, 8);
				}
			else if (c == 'n') {
				if (k >= argc)
					missing_arg(c, "uname handler");

				copy_string(config->unamehandler, argv[k++], sizeof(config->unamehandler));
				}
			else if (c == 'h') {
				if (k >= argc)
					missing_arg(c, "command handler");

				copy_string(config->handler, argv[k++], sizeof(config->handler));
				}
			else if (c == 'p')
				config->sgphack = 1;
			else if (c == 'q') {
				if (k >= argc)
					missing_arg(c, "home directory");

				copy_string(config->homedir, argv[k++], sizeof(config->homedir));
				}
			else if (c == 'r') {
				if (k >= argc)
					missing_arg(c, "virtual root");

				copy_string(config->vroot, argv[k++], sizeof(config->vroot));
				}
			else if (c == 's') {
				if (k >= argc)
					missing_arg(c, "sitehandler");

				copy_string(config->sitehandler, argv[k++], sizeof(config->sitehandler));
				}
			else if (c == 't') {
				if (k >= argc)
					missing_arg(c, "timeout");

				config->timeout = atoi(argv[k++]);
				if (config->timeout < 1)
					config->timeout = 60;
				}
			else if (c == 'u') {
				if (k >= argc)
					missing_arg(c, "userid");

				p = argv[k++];
				if (isalpha(*p)) {
					struct passwd *pw;

					if ((pw = getpwnam(p)) == NULL) {
						fprintf (stderr, "%s: no such user: %s\n", program, p);
						exit (1);
						}

					config->user.uid = pw->pw_uid;
					config->user.gid = pw->pw_gid;
					}
				else {
					char	*q;

					if (*p == '.') {
						config->user.uid = 65535U;
						q = p;
						}
					else
						config->user.uid = strtoul(p, &q, 10);

					if (*q == 0)
						config->user.gid = 65534U;
					else if (*q == '.') {
						q++;
						config->user.gid = strtoul(q, &q, 10);
						}
					else {
						fprintf (stderr, "%s: error in user id: %s\n", program, p);
						exit (1);
						}
					}
				}
			else if (c == 'v') {
				if (k >= argc)
					missing_arg(c, "varname prefix");

				copy_string(config->varname, argv[k++], sizeof(config->varname));
				}
			else if (c == 'w') {
				if (k >= argc)
					missing_arg(c, "denylist");

				copy_string(config->denylist, argv[k++], sizeof(config->denylist));
				}
			else if (c == 'y') {
				if (config->allow_anon != 0) {
					if (k >= argc)
						missing_arg(c, "anonymous user");

					copy_string(config->anon_user, argv[k++], sizeof(config->anon_user));
					}

				config->allow_anon = 1;
				}
			else if (c == 'D') {
				if (daemonmode == 1) {
					if (k >= argc)
						missing_arg(c, "server port");

					bindport = atoi(argv[k++]);
					}

				daemonmode = 1;
				}
			else if (c == 'V') {
				printf ("%s %s\n", argv[0], VERSION);
				exit (0);
				}
			else {
				fprintf (stderr, "%s: unknown option: -%c\n", program, c);
				exit (-1);
				}
			}
		}


	if (daemonmode != 0) {
		signal(SIGCHLD, SIG_IGN);
		if (bindport > 0) {
			int     sock;

			sock = bind_to_port("", bindport);
			acceptloop(sock);
			}
		}	


	ftp_session(config);
	exit (0);
}


