
/*

    File: site.c

    Copyright (C) 1998,1999  Wolfgang Zekoll  <wzk@quietsche-entchen.de>
    Copyright (c) 2000,2002,2003  Andreas Schoenberg  <asg@pce.de>
  
    This software is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
  
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
  
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

 */


#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <stdarg.h>
#include <errno.h>

#include <time.h>
#include <signal.h>
#include <wait.h>
#include <ctype.h>
#include <pwd.h>

#include <sys/types.h>
#include <sys/stat.h>
#include <sys/fcntl.h>
#include <sys/socket.h>
#include <netdb.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <syslog.h>
#include <sys/time.h>

#include "ftp.h"
#include "ip-lib.h"
#include "lib.h"


int getuniquename(ftp_t *x, char *filename, int size)
{
	int	rc, pid, pfd[2];
	char	line[300];


	*filename = 0;
	if (*x->config->unamehandler == 0)
		return (1);

	set_variables(x, "STOU", "", 0);

/*	getcwd(line, sizeof(line));
 *	setvar(x, "SYSDIR", line);
 *	setvar(x, "FTPDIR", get_ftppath(x, line, sizeof(line)));
 */

	/*
	 * Wir starten den SITE-Handler ...
	 */

	rc = 0;
	if (pipe(pfd) != 0) {
		syslog(LOG_NOTICE, "-ERR: can't pipe: %m");
		exit (1);
		}
	else if ((pid = fork()) < 0) {
		syslog(LOG_NOTICE, "-ERR: can't fork unamehandler: %m");
		exit (1);
		}
	else if (pid == 0) {
		int	argc;
		char	*argv[32];

		dup2(pfd[1], 1);	/* stdout nach FTP Client */
		close(pfd[0]);

		dup2(1, 2);		/* stderr auch */
		close(0);		/* kein stdin */

		copy_string(line, x->config->unamehandler, sizeof(line));
		argc = split(line, argv, ' ', 30);
		argv[argc] = NULL;
		execvp(argv[0], argv);

		syslog(LOG_NOTICE, "-ERR: can't exec unamehandler %s: %m", argv[0]);
		exit (1);
		}
	else {
		char	*p, line[300];
		FILE	*fp;

		/*
		 * Wir schliessen die nicht gebrauchten fd's ...
		 */

		close(pfd[1]);

		if ((fp = fdopen(pfd[0], "r")) == NULL)
			/* nichts */ ;
		else if (fgets(line, sizeof(line), fp) == NULL)
			/* auch nichts */ ;
		else {
			p = skip_ws(line);
			get_word(&p, filename, size);
			}

		fclose(fp);
		close(pfd[0]);
		if (waitpid(pid, &rc, 0) < 0) {
			syslog(LOG_NOTICE, "-ERR: error while waiting for unamehandler: %m");
			exit (1);
			}
		}

	if (*filename == 0)
		return (1);

	return (0);
}


int dosite(ftp_t *x, char *par)
{
	int	c, d, i, k;
	int	rc, pid, pfd[2];
	char	line[300];


	if (*x->config->sitehandler == 0) {
		cfputs(x, "502 command not implemented");
		return (0);
		}


	set_variables(x, "SITE", par, 1);

	/*
	 * In der SITE-Parameterzeile aufeinanderfolgende Leerzeichen
	 * zusammenziehen ...
	 */

	k = d = 0;
	for (i=0; (c = x->sitepar[i]) != 0; i++) {
		if (c < ' ')
			c = ' ';

		if (c == ' '  &&  d == ' ')
			/* nichts, ueberspringen */ ;
		else {
			x->sitepar[k++] = c;
			d = c;
			}
		}
		
	setvar(x, "ARG", x->sitepar);
	if (debug != 0)
		fprintf (stderr, "sitepar= %s\n", x->sitepar);


	/*
	 * ... und fuer akanga shell als Array zur Verfuegung stellen.
	 */

	for (i=0; (c = x->sitepar[i]) != 0; i++) {
		if (c == ' ')
			x->sitepar[i] = '\001';
		}

	setvar(x, "ARGV", x->sitepar);



	/*
	 * Wir starten den SITE-Handler ...
	 */

	rc = 0;
	if (pipe(pfd) != 0) {
		syslog(LOG_NOTICE, "-ERR: can't pipe: %m");
		exit (1);
		}
	else if ((pid = fork()) < 0) {
		syslog(LOG_NOTICE, "-ERR: can't fork sitehandler: %m");
		exit (1);
		}
	else if (pid == 0) {
		int	argc;
		char	*argv[32];

		dup2(pfd[1], 1);	/* stdout nach FTP Client */
		close(pfd[0]);

		dup2(1, 2);		/* stderr auch */
		close(0);		/* kein stdin */

		copy_string(line, x->config->sitehandler, sizeof(line));
		argc = split(line, argv, ' ', 30);
		argv[argc] = NULL;
		execvp(argv[0], argv);

		syslog(LOG_NOTICE, "-ERR: can't exec sitehandler %s: %m", argv[0]);
		exit (1);
		}
	else {
		int	rc, statuscode;
		char	*p, line[300];
		FILE	*fp;

		/*
		 * Wir schliessen die nicht gebrauchten fd's ...
		 */

		close(pfd[1]);

		if ((fp = fdopen(pfd[0], "r")) == NULL)
			cfputs(x, "500 error");
		else {
			statuscode = 0;
			while (1) {
				if (fgets(line, sizeof(line), fp) == NULL) {
					if (statuscode == 0)
						cfputs(x, "500 error");
					else {
						snprintf (line, sizeof(line) - 2, "%03d ok", statuscode);
						cfputs(x, line);
						}
					}

				noctrl(line);
				cfputs(x, line);
				rc = strtoul(line, &p, 10);
				if (rc > 100  &&  *p == ' ')
					break;

				if (statuscode == 0) {
					if (rc > 100  &&  rc < 1000)
						statuscode = rc;
					}
				}
			}

		close(pfd[0]);
		if (waitpid(pid, &rc, 0) < 0) {
			syslog(LOG_NOTICE, "-ERR: error while waiting for sitehandler: %m");
			exit (1);
			}
		}

	return (0);
}

